/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "CommandData.h"
#include "ICommand.h"

CommandData::CommandData(const QString& line, PredicateList argp, bool repeatLastPredicate)
{
    for (int i = 0, ppos = 0; i < line.length(); ++i) {
        QChar c = line[i];
        if (c == ' ')
            continue;
        else {
            if (ppos >= static_cast<int>(argp.size()) && m_l.at(ppos-1)) {
                if (repeatLastPredicate)
                    ppos = argp.size()-1;
                else
                    break;
            }

            const auto& result = argp[ppos](i, line);
            m_l.push_back(result);
            ++ppos;
            if (!result)
                --i;
        }
    }

    while (m_l.size() < argp.size())
        m_l.push_back(std::nullopt);
}

int CommandData::size() const
{
    return m_l.size();
}

const std::optional<std::string>& CommandData::operator[](int idx)
{
    return m_l.at(idx);
}

std::string CommandData::joinString(char sep) const
{
    std::string ret;
    for (const auto& item : m_l) {
        if (!ret.empty())
            ret += sep;
        if (item.has_value())
            ret += *item;
    }
    return ret;
}
