/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (c) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "ICommand/ICommandPriv.h"
#include "IRCClient/DCC.h"
#include <fmt/format.h>

void ICommandPriv::cmd_dcc(const std::string& command, const std::string& target, const std::string& message)
{
    auto& mdi = MdiManager::instance();

    if (command == "CHAT") {
        auto result = connection.initiateDCC();
        if (result.second != IRCError::NoError) {
            const auto code = static_cast<int>(result.second);
            const auto errorStr = QString::fromStdString(IRCErrorToString(result.second));
            mdi.currentStatus()->printToActive(PrintType::ProgramInfo, QObject::tr("/DCC: Unable to initiate DCC Chat to %1 [%2 (%3)]")
                                                                       .arg( QString::fromStdString(target), errorStr, QString::number(code)) );
        }
        else {
            auto dcc = result.first;
            const auto portno = dcc->isReversed() ? 0
                                                  : dcc->port();

            auto* window = mdi.createDCCSubwindow(mdi.currentStatus(), IWin::Type::DCCChat, result.first, IRCPrefix(target));

            if (dcc->isReversed())
                window->print(PrintType::ProgramInfo, QObject::tr("Initiating CHAT with %1, please wait...")
                                                        .arg( QString::fromStdString(target) ));
            else
                window->print(PrintType::ProgramInfo, QObject::tr("Initiating CHAT with %1 on port %2, please wait...")
                        .arg( QString::fromStdString(target) )
                        .arg(portno));

            const auto myIp = findOwnIpAddress();
            const auto msg = fmt::format( "CHAT chat {} {}",
                                          normalIpToLong(myIp),
                                          portno
                                        );

            cmd_ctcp(target, "DCC", msg);
        }
    }
    else {
        mdi.currentStatus()->printToActive(PrintType::ProgramInfo, QObject::tr("/DCC: Unknown DCC method: %1")
                                                                    .arg( QString::fromStdString(command) ));
    }
}
