/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "AddServer.h"
#include "ui_AddServer.h"
#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>

AddServer::AddServer(const QStringList& networkList, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AddServer)
{
    ui->setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    auto EnableOrDisableSave = [this](const QString&) {
        auto isDisabled = ui->edName->text().isEmpty() || ui->edAddress->text().isEmpty();
        ui->btnSave->setEnabled(!isDisabled);
    };

    connect(ui->edName, &QLineEdit::textChanged, EnableOrDisableSave);
    connect(ui->edAddress, &QLineEdit::textChanged, EnableOrDisableSave);

    for (const auto& networkName : networkList)
        ui->edNetwork->addItem(networkName);
}

AddServer::~AddServer()
{
    delete ui;
}

QString AddServer::name() const
{
    return ui->edName->text();
}

QString AddServer::address() const
{
    return ui->edAddress->text();
}

QString AddServer::password() const
{
    return ui->edPassword->text();
}

QString AddServer::sasl() const
{
    return ui->edSASL->text();
}

bool AddServer::ssl() const
{
    return ui->chkSSL->isChecked();
}

bool AddServer::isServer() const
{
    return ui->rdServer->isChecked();
}

int AddServer::networkIndex() const
{
    // 0th index is the "No network" selection, make that into a "-1" index instead... and the rest must also follow.
    return ui->edNetwork->currentIndex() - 1;
}

void AddServer::on_btnCancel_clicked()
{
    close();
}

void AddServer::on_btnSave_clicked()
{
    emit saved();
    close();
}

void AddServer::on_rdNetwork_toggled(bool checked)
{
    ui->edNetwork->setEnabled(!checked);

    // Network servers (child servers) uses their parent server's SASL.
    // The individual child servers will not have own SASL credentials.
    ui->edSASL->setEnabled(!checked && ui->edNetwork->currentIndex() == 0 || checked);
}

void AddServer::on_edNetwork_currentIndexChanged(int index)
{
    ui->edSASL->setEnabled(index == 0);
}
