/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef ICONFIG_H
#define ICONFIG_H

#include "IConfigServers.h"
#include "IConfigOptions.h"
#include "IConfigLogging.h"
#include <QDialog>
#include <QSignalMapper>
#include <QHBoxLayout>
#include <QCloseEvent>

namespace Ui {
class IConfig;
}

class IConfig : public QDialog
{
    Q_OBJECT

public:
    explicit IConfig(QWidget *parent = nullptr);
    ~IConfig();
    void showDisconnectButton();
    void hideDisconnectButton();

private slots:
    void on_btnSave_clicked();
    void on_btnSaveConnect_clicked();
    void on_btnDisconnect_clicked();
    void on_btnClose_clicked();

private:
    enum class SubDialogType {
        Servers,
        Options,
        Logging
    };
    void closeEvent(QCloseEvent* evt);
    void showSubDialog(SubDialogType dlg);
    void saveAll();
    bool askForSave();

    Ui::IConfig* ui;
    QHBoxLayout *layout;
    IConfigServers* servers;
    IConfigOptions* options;
    IConfigLogging* logging;

signals:
    void connectToServer(bool newServer);
    void disconnectFromServer();
};

#endif // ICONFIG_H
