/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef ICONFIGSERVERS_H
#define ICONFIGSERVERS_H

#include "ServerModel.h"
#include "ServerOptionsDelegate.h"
#include <QWidget>

namespace Ui {
class IConfigServers;
}

class AddServer;

class IConfigServers : public QWidget
{
    Q_OBJECT

public:
    explicit IConfigServers(QWidget *parent = nullptr);
    ~IConfigServers();
    bool isChanged() const;
    bool connectToNewStatus() const;
    void unsetConnectToNewStatus();
    void save();
    void reset();

private slots:
    void on_btnShowPassword_toggled(bool checked);
    void on_btnShowSASL_toggled(bool checked);
    void on_servers_clicked(const QModelIndex& index);
    void on_btnAddServer_clicked();
    void on_btnDeleteSelected_clicked();

private:
    Ui::IConfigServers *ui;
    AddServer* addServerDlg{};
    ServerOptionsDelegate m_serverOptionsDelegate;
    ServerModel smodel;
    QString cf_Realname;
    QString cf_Username;
    QString cf_Nickname;
    QString cf_AltNickame;
    QString cf_Server;
    QString cf_Password;
    QString cf_SASL;
    bool cf_SSL;
    bool serversModified{ false };
};

#endif // ICONFIGSERVERS_H
