/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "ServerItem.h"

namespace Key = ServerJsonKey;

ServerItem::ServerItem(const QJsonObject& data, int row, ServerItem* parentItem, bool isNetwork)
    : m_data(data)
    , m_row(row)
    , m_parent(parentItem)
    , m_isNetwork{ isNetwork }
{}

QString ServerItem::name() const
{
    return m_data[Key::Name].toString();
}

QString ServerItem::address() const
{
    auto host{ m_data[Key::Host].toString() };
    auto port{ m_data[Key::Port].toInt() };

    if (host.isEmpty())
        host = "no.host";

    if (port == 0)
        port = ssl() ? 6697 : 6667;

    return QStringLiteral("%1:%2").arg(host).arg(port);
}

QString ServerItem::password() const
{
    return m_data[Key::Password].toString();
}

QString ServerItem::sasl() const
{
    return m_data[Key::SASL].toString();
}

bool ServerItem::ssl() const
{
    return m_data[Key::SSL].toBool();
}
