/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef SERVERITEM_H
#define SERVERITEM_H

#include <QList>
#include <QString>
#include <QJsonObject>

namespace ServerJsonKey
{
constexpr auto Servers  = "Servers";
constexpr auto Networks = "Networks";
constexpr auto Name     = "Name";
constexpr auto Host     = "Host";
constexpr auto Port     = "Port";
constexpr auto Password = "Password";
constexpr auto SASL     = "SASL";
constexpr auto SSL      = "SSL";
}

class ServerItem
{
public:
    ServerItem(const QJsonObject& data, int row, ServerItem* parentItem, bool isNetwork);

    QString name() const;
    void setName(const QString& name) { m_data[ServerJsonKey::Name] = name; }

    QString address() const;
    void setAddress(const QString& address)
    {
        auto hostport = address.split(':');

        if (hostport.size() == 1)
            hostport << (ssl() ? "6697" : "6667");

        if (hostport[0].isEmpty())
            hostport[0] = "server.name";

        m_data[ServerJsonKey::Host] = hostport[0];
        m_data[ServerJsonKey::Port] = hostport[1].toUShort();
    }

    QString password() const;
    void setPassword(const QString& password) { m_data[ServerJsonKey::Password] = password; }

    QString sasl() const;
    void setSasl(const QString& credentials) { m_data[ServerJsonKey::SASL] = credentials; }

    bool ssl() const;
    void setSsl(bool enable) { m_data[ServerJsonKey::SSL] = enable; }

    ServerItem* parent() const { return m_parent; }
    int row() const { return m_row; }
    void incRow() { ++m_row; }
    void decRow() { --m_row; }

    QList<ServerItem>& children() { return m_children; }

    bool isNetwork() const { return m_isNetwork; }

private:
    QJsonObject m_data;
    QList<ServerItem> m_children;
    int m_row;
    ServerItem* m_parent;
    bool m_isNetwork;
};

#endif // SERVERITEM_H
