/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBase.h"
#include "../IRCBasePriv.h"

void IRCBasePriv::cmdJoin(const IRCMessage& ircmessage)
{
    const auto& msg = ircmessage.getMessage();
    const auto& sender = ircmessage.getSender();

    if (ircmessage.getArgs().size() == 2) {
        std::string accountname = ircmessage[1];
        if (accountname[0] == '*')
            accountname.clear();

        /* We are joining a channel */
        if (sender.nickname() == nickname) {
            auto chanentry = super.getChannel(ircmessage[0]);
            if (!chanentry)
                channels.emplace_back(std::make_shared<IRCChannel>(ircmessage[0], super));
        }
        else
            addMemberToChannel(sender, ircmessage[0]);
        super.v3onMsgJoin(sender, ircmessage[0], accountname, msg);
    }
    else {
        /* We are joining a channel */
        auto channelName = msg.empty() ? ircmessage[0] : msg;
        if (sender.nickname() == nickname) {
            auto chanentry = super.getChannel(channelName);
            if (!chanentry)
                channels.emplace_back(std::make_shared<IRCChannel>(channelName, super));
        }
        else
            addMemberToChannel(sender, channelName);
        super.onMsgJoin(sender, channelName);
    }
}