/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBase.h"
#include "../IRCBasePriv.h"
#include "../Utilities.h"

#include <iostream> // Temporary printouts for making DCC work.

void IRCBasePriv::cmdPrivmsg(const IRCMessage& ircmessage)
{
    const auto& sender = ircmessage.getSender();
    const auto& msg = ircmessage.getMessage();

    if (msg[0] == CTCPflag) {
        auto[cmd,ctcpMsg] = FormatCTCPLine(msg);

        if (cmd == "DCC") {
            auto[dccCmd,dccMsg] = FormatCTCPLine(ctcpMsg);
            std::vector<std::string> dccMsgTks;
            {
                std::stringstream ss(dccMsg);
                dccMsgTks = std::vector<std::string>(std::istream_iterator<std::string>{ss},
                                                     std::istream_iterator<std::string>());
            }

            if (dccCmd == "CHAT" && dccMsgTks.size() >= 3 && dccMsgTks[0] == "chat") {
                const auto& dccLongIp = dccMsgTks[1];
                const auto& dccPort = dccMsgTks[2];
                if (port == "0") {
                    // nat stuff
                    std::cout << "DCC CHAT TODO" << std::endl;
                }
                else {
                    const auto ip = longIpToNormal(dccLongIp);
                    dcclist.emplace_back(std::make_shared<DCC>(super, ioctx, ip, dccPort));

                    super.onMsgDCCRequest(dcclist.back(), sender, ircmessage[0], dccCmd, dccMsg);
                }
            }
        }
        else
            super.onMsgCTCPRequest(sender, ircmessage[0], cmd, ctcpMsg);
    }
    else
        super.onMsgPrivmsg(sender, ircmessage[0], msg);
}
