/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBase.h"
#include "../IRCBasePriv.h"

void IRCBasePriv::cmdQuit(const IRCMessage& ircmessage)
{
    std::vector<std::shared_ptr<IRCChannel>> channelsAffected;
    std::vector<std::string> channelsAffectedStr;

    auto member = super.getMember(ircmessage.getSender().toString());
    if (member) {
        const auto& chans = member->channels();
        for (const auto& c : chans) {
            channelsAffected.push_back(c.lock());
            channelsAffectedStr.push_back(c.lock()->name());
        }

        auto it = std::find(allMembers.begin(), allMembers.end(), member);
        allMembers.erase(it);
    }

    super.onMsgQuit(ircmessage.getSender(), ircmessage.getMessage(), channelsAffectedStr);

    for (const auto& c : channelsAffected)
        c->delMember(member);
}
