/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IRCCHANNEL_H
#define IRCCHANNEL_H

#include "IRCMemberEntry.h"
#include <memory>
#include <string>
#include <vector>
#include <unordered_map>
#include <optional>

class IRCMember;
class IRCBase;

using IRCMemberEntryRef = std::reference_wrapper<IRCMemberEntry>;

class IRCChannel
{
public:
    IRCChannel(const std::string& name, IRCBase& owner);
    ~IRCChannel() = default;

    const std::string& name() const;

    std::optional<IRCMemberEntryRef> getMember(const std::string& nickname);
    IRCMemberEntry& addMember(std::shared_ptr<IRCMember> member);
    void delMember(std::shared_ptr<IRCMember> member);
    const std::vector<IRCMemberEntry>& members() const;

    void setTopic(const std::string& topic);
    const std::string& topic() const;

    const std::unordered_map<char, std::string>& modes() const;
    void setMode(char m, const std::string& parameter = "");
    void delMode(char m);

    bool isPopulating() const;
    void donePopulating();

private:
    std::string m_name;
    std::vector<IRCMemberEntry> m_members;
    std::string m_topic;
    std::unordered_map<char, std::string> m_modes;
    bool m_populating{ true };
    IRCBase* m_owner; // TODO  use reference?
};


#endif // IRCCHANNEL_H
