/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IRCERROR_H
#define IRCERROR_H

#include <fmt/format.h>
#include <system_error>
#include <string>

extern int lastAsioErrorCode;
extern std::string lastAsioErrorMessage;

enum class IRCError
{
    NoError,

    NotConnected,
    AlreadyConnected,

    CannotResolveAddress,
    BrokenPipe,
    ConnectionAborted,
    ConnectionRefused,
    ConnectionReset,
    HostUnreachable,
    NetworkDown,
    NetworkReset,
    NetworkUnreachable,
    NoDescriptors,
    TimedOut,
    EndOfFile,

    SSL_SelfSigned,
    SSL_CN_Mismatch,
    SSL_CN_Missing,
    SSL_CN_WildcardIllegal,
    SSL_NotYetValid,
    SSL_Expired,

    CannotChangeWhenConnected,
    HostNotSet,
    PortNotSet,
    IdentNotSet,
    NicknameNotSet,
    RealnameNotSet,

    DCC_NotATarget,
    DCC_TimedOut,
    DCC_InitiateError,

    UnhandledException
};

std::string IRCErrorToString(IRCError e);
IRCError SystemErrorToIRCError(const std::system_error& e);

template <> struct fmt::formatter<IRCError> : formatter<string_view> {
    template <typename FormatContext>
    auto format(IRCError e, FormatContext& ctx) {
        auto txt = IRCErrorToString(e);
        return formatter<string_view>::format(txt, ctx);
    }
};

#endif // IRCERROR_H
