/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IRCMEMBER_H
#define IRCMEMBER_H

#include "IRCPrefix.h"
#include <vector>
#include <memory>
#include <string>

class IRCChannel;

class IRCMember
{
public:
    explicit IRCMember(const IRCPrefix& prefix);
    explicit IRCMember(const std::string& nickname);

    const IRCPrefix& prefix() const;
    void setPrefix(const IRCPrefix& prefix);
    const std::vector<std::weak_ptr<IRCChannel>>& channels();
    void addChannel(std::weak_ptr<IRCChannel> channel);
    void delChannel(std::weak_ptr<IRCChannel> channel);

    void setNickname(const std::string& nickname);

    bool isDangling() const { return m_channels.empty(); }

private:
    IRCPrefix m_prefix;
    std::vector<std::weak_ptr<IRCChannel>> m_channels;
};

#endif // IRCMEMBER_H
