/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IRCMESSAGE_H
#define IRCMESSAGE_H

#include "IRCPrefix.h"

#include <string>
#include <vector>
#include <chrono>

class IRCMessage
{
public:
    IRCMessage(const IRCPrefix& defaultSender, const std::string& message);

    const std::vector<std::string>& getTags() const { return m_tags; }
    const IRCPrefix& getSender() const { return m_sender; }
    const std::string& getCommand() const { return m_command; }
    const std::vector<std::string>& getArgs() const { return m_args; }
    const std::string& getMessage() const { return m_message; }
    const std::string& getBatchId() const { return m_batchId; }
    const std::chrono::system_clock::time_point& getTimestamp() const { return m_timestamp; }

    bool isNumeric() const { return !m_command.empty() && m_command[0] >= '0' && m_command[0] <= '9'; }

    bool operator==(const char* commandName) const { return m_command == commandName; }
    const std::string& operator[](int idx) const;

private:
    using SystemClock = std::chrono::system_clock;

    std::vector<std::string> m_tags; //!< IRCv3 tags
    IRCPrefix m_sender; //!< Sender prefix
    std::string m_command; //!< Command name
    std::vector<std::string> m_args; //!< Arguments to command
    std::string m_message; //!< Message to command

    std::string m_batchId; //!< Set if this is part of a batch
    SystemClock::time_point m_timestamp; //!< When message occurred.

    void parseIRCv3Tags(const std::string& tagstr);
};

#endif // IRCMESSAGE_H
