/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBasePriv.h"

void IRCBasePriv::parseChannelModeMessage(std::shared_ptr<IRCChannel> chan, const std::string& modes, const std::vector<std::string>& args) const
{
    char sign; // + or -
    int argidx = 0;
    for (char m : modes) {
        if (m == '+' || m == '-') {
            sign = m;
            continue;
        }
        char group = channelModeGroup(m);
        if (group == 'A')
            ++argidx;

        else if (group == 'B') {
            if (sign == '+')
                chan->setMode(m, args[argidx]);
            else
                chan->delMode(m);
            ++argidx;
        }

        else if (group == 'C') {
            if (sign == '+') {
                chan->setMode(m, args[argidx]);
                ++argidx;
            }
            else
                chan->delMode(m);
        }

        else if (group == 'D') {
            if (sign == '+')
                chan->setMode(m, "");
            else
                chan->delMode(m);
        }

        else if (group == 'M') {
            auto member = chan->getMember(args[argidx]);
            ++argidx;
            if (sign == '+')
                member->get().addMode(m);
            else
                member->get().delMode(m);
        }
    }
}
