/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBasePriv.h"

void IRCBasePriv::read(const asio::error_code& ec, std::size_t /*size*/)
{
    lastError = SystemErrorToIRCError(ec);
    if (lastError != IRCError::NoError) {
        disconnectHandler();
        return;
    }

    auto crIt = std::find(readbuf.begin(), readbuf.end(), '\r');

    if (crIt != readbuf.end()) {
        std::string line(readbuf.begin(), crIt);
        readbuf.erase(readbuf.begin(), crIt + 2);
        parseIncoming(line);
    }

    if (useSSL) {
        asio::async_read_until(sslsock.value(), asio::dynamic_buffer(readbuf), "\r\n",
                               [this](const asio::error_code& e, std::size_t size){ read(e, size);	});
    }
    else {
        asio::async_read_until(sock.value(), asio::dynamic_buffer(readbuf), "\r\n",
                               [this](const asio::error_code& e, std::size_t size){ read(e, size);	});
    }
}
