/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBasePriv.h"
#include <fmt/format.h>
#include <iostream> // Needs ties with a centralised logger... spdlog :)

void IRCBasePriv::write(const std::string& command, const std::vector<std::string>& args, const std::string& msg)
{
    std::string out = command;

    if constexpr (DumpWriteData)
        std::cout << fmt::format(R"([SND] cmd="{}" msg="{}" argc={})", command, msg, args.size());

    int c = 0;
    for (const auto& arg : args) {
        out.append(" " + arg);
        if constexpr (DumpWriteData) {
            std::cout << fmt::format(R"( arg({})="{}")", c, arg);
            ++c;
        }
    }

    if constexpr (DumpWriteData)
        std::cout << std::endl;

    out.append(" :" + msg);
    out.append("\r\n");

    if (useSSL)
        sslsock->write_some(asio::buffer(out));
    else
        sock->write_some(asio::buffer(out));
}

void IRCBasePriv::write(const std::string& command, const std::string& msg)
{
    std::string out = fmt::format("{} :{}\r\n", command, msg);
    if constexpr (DumpWriteData) {
        std::cout << fmt::format(R"([SND] cmd="{}" msg="{}")", command, msg) << std::endl;
    }
    if (useSSL)
        sslsock->write_some(asio::buffer(out));
    else
        sock->write_some(asio::buffer(out));
}
