/*
 * IdealIRC Core - Internet Relay Chat API
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "../IRCBasePriv.h"

#include <fmt/format.h>

#include <iostream> // maybe find a way to collect all "stdout" to a logger library... spdlog is nice :)

void IRCBasePriv::writeNoMsg(const std::string& command, const std::vector<std::string>& args)
{
    std::string out = command;

    if constexpr (DumpWriteData)
        std::cout << fmt::format(R"([SND] cmd="{}" argc={})", command, args.size());

    int c = 0;
    for (const auto& arg : args) {
        out.append(" " + arg);
        if constexpr (DumpWriteData) {
            std::cout << fmt::format(R"( arg({})="{}")", c, arg);
            ++c;
        }
    }

    if constexpr (DumpWriteData)
        std::cout << std::endl;

    out.append("\r\n");

    if (useSSL)
        sslsock->write_some(asio::buffer(out));
    else
        sock->write_some(asio::buffer(out));
}
