/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IWIN_H
#define IWIN_H

#include "Widgets/IIRCView.h"
#include <QWidget>
#include <unordered_map>
#include <QCloseEvent>
#include <QShowEvent>

class IWin : public QWidget
{
    Q_OBJECT

public:
    enum class Type
    {
        Undefined,
        Status,
        Channel,
        Private,
        DCCChat,
        Custom
    };
    static const QString InvalidWindowTypeString;
    static std::unordered_map<Type,QString> TypeString;

    Type getType() const;
    IWin* getParent() const { return m_parent; }
    IWin* getStatusParent();

    bool print(PrintType ptype, const QString& text);
    virtual bool printWithCustomTime(const QDateTime& timestamp, PrintType ptype, const QString& text) = 0;
    virtual void refreshWindowTitle() = 0;
    virtual void clear() = 0;

    const QString& getButtonText() const;
    void setButtonText(const QString& text);

protected:
    explicit IWin(Type type, IWin* parentWindow = nullptr);
    void closeEvent(QCloseEvent*) override;

private:
    Type m_type;
    QString m_buttonText;
    IWin* m_parent{ nullptr };
    bool m_firstShow{ true };

signals:
    void aboutToClose(IWin* who);
};

#endif // IWIN_H
