/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IWINCHANNEL_H
#define IWINCHANNEL_H

#include "IWin.h"
#include "NicklistController.h"
#include "Widgets/ILineEdit.h"
#include "Widgets/IListWidget.h"
#include "Script/SymbolScope.h"
#include <QSplitter>
#include <QVBoxLayout>
#include <QListWidget>
#include <QTableView>
#include <QCloseEvent>
#include <memory>

class IWinStatus;

class IWinChannel : public IWin
{
    Q_OBJECT

public:
    IWinChannel(IWinStatus* statusParent, const QString& channelName);

    bool printWithCustomTime(const QDateTime& timestamp, PrintType ptype, const QString& text) override;
    void refreshWindowTitle() override;
    void clear() override { view->resetView(); }
    void resetNicklist();

private:
    void newLine(const QString& line);
    void listboxDoubleclick(QListWidgetItem* item);
    bool isUserModePrefix(QChar c); //! Returns true if the passed character is a prefix like @ + etc

    void closeEvent(QCloseEvent* evt) override;
    int tabComplete(int index, QString& pattern);

    QSplitter* splitter;
    QVBoxLayout* v_layout;
    IIRCView* view;
    IListWidget* listbox;
    ILineEdit* input;

    NicklistController* nlControl;

    IWinStatus* status;
    IRC& connection;

    SymbolScope menuSymbols;

private slots:
    void memberListReloaded(const QString& channel);
    void memberListReset(const QString& channel);
    void memberListCleared();
    void memberListAdd(const QString& channel, const IRCMemberEntry& member);
    void memberListUpdateMember(const QString& nickname, const IRCMemberEntry& member);
    void memberListRemoveMember(const QString& channel, const IRCMemberEntry& member);
};

#endif // IWINCHANNEL_H
