/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IWINDCCCHAT_H
#define IWINDCCCHAT_H

#include "IWin.h"
#include "IRCClient/DCC.h"
#include "Widgets/ILineEdit.h"
#include <QVBoxLayout>
#include <QCloseEvent>
#include <memory>

class IWinStatus;

class IWinDCCChat : public IWin
{
    Q_OBJECT

public:
    IWinDCCChat(IWinStatus* statusParent, std::shared_ptr<DCC> dcc_, const QString& targetNickname);

    bool printWithCustomTime(const QDateTime& timestamp, PrintType ptype, const QString& text) override;
    void refreshWindowTitle() override;
    void clear() override { view->resetView(); }

    void acceptAsTarget();
    void disconnectFromDCC();

    void markReverseInitiatePending() { reverseInitiatePending = true; }
    bool isReverseInitiatePending() const { return reverseInitiatePending; }
    void initiateReversed(std::shared_ptr<DCC> dcc_);

private:
    void setupCallbacks();
    void closeEvent(QCloseEvent* evt) override;
    void onConnected();
    void onDisconnected(IRCError e);
    void newLine(const QString& line);
    void onRead(const DCC::ByteString& data, std::size_t length);
    int tabComplete(int index, QString& pattern);

    QVBoxLayout* layout;
    IIRCView* view;
    ILineEdit* input;
    QString lineBuffer;

    bool reverseInitiatePending{ false };
    std::shared_ptr<DCC> dcc;
};


#endif // IWINDCCCHAT_H
