/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IWINSTATUS_H
#define IWINSTATUS_H

#include "IWin.h"
#include "IRC.h"
#include "InputHandler.h"
#include "ButtonbarMgr.h"
#include "Widgets/ILineEdit.h"
#include "Script/SymbolScope.h"
#include <QLineEdit>
#include <QVBoxLayout>
#include <QTimer>

class MdiManager;
class IWinChannel;
class IWinDCCChat;

class IWinStatus : public IWin
{
    Q_OBJECT

public:
    explicit IWinStatus(const QString& buttonText);

    bool printWithCustomTime(const QDateTime& timestamp, PrintType ptype, const QString& text) override;
    void refreshWindowTitle() override;
    void clear() override { view->resetView(); }
    void printTo(const QString& target, const PrintType ptype, const QString& text);
    void printToTypes(const IWin::Type toType, const PrintType ptype, const QString& text);
    void printToActive(const PrintType ptype, const QString& text); // TODO  test
    void printToAll(const PrintType ptype, const QString& text);
    IWinChannel* createChannelWindow(const QString& name);
    IWin* createPrivateWindow(const QString& target, bool setFocus = true, Highlight buttonHighlight = HL_None);
    IWinDCCChat* createDCCChatWindow(std::shared_ptr<DCC> dcc, const IRCPrefix& sender);
    IWin* getActiveWindow();
    QList<IWin*> subWindows() const;
    IRC& getConnection() { return connection; }
    const IRC& getConnection() const { return connection; }
    ILineEdit& getInputBox() { return *input; }

private:
    void closeEvent(QCloseEvent* evt) override;
    void newLine(const QString& line);
    void closeAllChildren();
    void Qt_Eventloop_Hook();
    void disconnectedFromServer();
    int tabComplete(int index, QString& pattern);

    static int StatusWindowCount;
    QVBoxLayout* layout;
    IIRCView* view;
    ILineEdit* input;

    IRC connection;

    SymbolScope menuSymbols;
};

#endif // IWINSTATUS_H
