/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef NICKLISTCONTROLLER_H
#define NICKLISTCONTROLLER_H

#include <QObject>
#include <QListWidget>
#include <memory>
#include <optional>

class IRC;
class IRCChannel;
class IRCMemberEntry;

class NicklistController : public QObject
{
    Q_OBJECT

public:
    NicklistController(QListWidget* listbox, IRC& server, const QString& channel, QObject* parent = nullptr);
    void reload();
    void clear();
    void addMember(const IRCMemberEntry& member);
    void updateMember(const QString& nickname, const IRCMemberEntry& member);
    void removeMember(const IRCMemberEntry& member);

private:
    bool lessThan(const QString& left, const QString& right);
    QString makeItemText(const IRCMemberEntry& member);
    int findNewItemPosition(const QString& text);
    void updatePrefixColor(QListWidgetItem* item, std::optional<QColor> color);
    void newConfiguration();
    QListWidget* m_listbox;
    IRC& m_server;
    const QString m_channel;
    QStringList m_items;
    QString m_prefixOrdered; //! Nickname prefix (@%+) in order
};

#endif // NICKLISTCONTROLLER_H
