/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef CONFIGMGR_H
#define CONFIGMGR_H

#include "IniFile.h"
#include <QObject>
#include <QString>
#include <QStringList>
#include <QHash>
#include <QVector>
#include <QColor>
#include <optional>
#include <unordered_map>

class ConfigMgr : public QObject
{
    Q_OBJECT

public:
    static ConfigMgr& instance();
    void save();

    QString geometry(const QString& key) const;
    QString connection(const QString& key) const;
    QString common(const QString& key) const;
    QString color(const QString& key) const;
    QString window(const QString& key) const;
    QString logging(const QString& key) const;
    QString IRCv3(const QString& key) const;
    std::optional<QColor> prefixColor(const QChar& prefix) const;
    QHash<QString,QString> color() const;
    QVector<std::pair<QChar,QColor>> prefixColor() const; // QVector of pairs preserves the order.
    QStringList scripts() const;

    void setGeometry(const QString& key, const QString& value);
    void setConnection(const QString& key, const QString& value);
    void setCommon(const QString& key, const QString& value);
    void setWindow(const QString& key, const QString& value);
    void setLogging(const QString& key, const QString& value);
    void setIRCv3(const QString& key, const QString& value);
    void setColorPalette(const QHash<QString,QString>& palette);
    void setPrefixColorPalette(const QVector<std::pair<QChar,QColor>>& palette);
    void addScript(const QString& path);
    void delScript(const QString& path);

private:
    ConfigMgr();
    QString genericConfigGetter(const QString& section, const QString& key, const std::unordered_map<std::string,std::string>& defaults) const;
    QString genericConfigGetter(const QString& section, const QString& key) const;
    IniFile ini;

signals:
    void saved();
};

#endif // CONFIGMGR_H
