/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "DCCQuery.h"
#include "ui_DCCQuery.h"
#include "MdiManager.h"

#include "IWin/IWinStatus.h"
#include "IWin/IWinDCCChat.h"

DCCQuery::DCCQuery(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::DCCQuery)
{
    ui->setupUi(this);
    ui->queryTable->setModel(&m_model);
    ui->queryTable->setItemDelegateForColumn(2, &m_delegate);
    ui->queryTable->setItemDelegateForColumn(3, &m_delegate);

    connect(&m_delegate, &DCCQueryDelegate::accept, this, &DCCQuery::accepted);
    connect(&m_delegate, &DCCQueryDelegate::reject, this, &DCCQuery::rejected);
}

DCCQuery::~DCCQuery()
{
    delete ui;
}

void DCCQuery::on_btnRejectAll_clicked()
{
    m_model.removeAll();
}

void DCCQuery::accepted(const QModelIndex& index)
{
    auto setupChatWindow = [](IWin* base) {
        auto* window = dynamic_cast<IWinDCCChat*>(base);
        if (!window)
            return;

        window->acceptAsTarget();
    };

    auto& mdi = MdiManager::instance();
    auto dcc = m_model.getDCCPtr(index);
    auto sender = m_model.getSender(index);
    auto& context = dynamic_cast<IRC&>(dcc->context());

    auto* basewin = mdi.createDCCSubwindow(&(context.getStatus()), IWin::Type::DCCChat, dcc, *sender);

    setupChatWindow(basewin);

    m_model.removeEntry(index);
}

void DCCQuery::rejected(const QModelIndex& index)
{
    m_model.removeEntry(index);
}

void DCCQuery::addQuery(const IRCPrefix& sender, const QString& description, const std::shared_ptr<DCC>& dcc)
{
    auto buttonPair = m_model.addQuery(sender, description, dcc);
    ui->queryTable->openPersistentEditor(buttonPair.first);
    ui->queryTable->openPersistentEditor(buttonPair.second);
    ui->queryTable->resizeColumnToContents(1); // Always make second column view the contents; for DCC SEND, showing of filename, etc.
}
