/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (c) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef DCCQUERYTABLEMODEL_H
#define DCCQUERYTABLEMODEL_H

#include "IRC.h"
#include "IRCClient/DCC.h"
#include <QAbstractTableModel>
#include <vector>
#include <memory>
#include <optional>
#include <utility>

class DCCQueryTableModel : public QAbstractTableModel
{
    Q_OBJECT

public:
    explicit DCCQueryTableModel(QObject* parent = nullptr);

    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex& /*child*/) const override { return QModelIndex{}; };

    int rowCount(const QModelIndex& parent = QModelIndex()) const override;
    int columnCount(const QModelIndex& parent = QModelIndex()) const override { return 4; }; // Sender, Description, Accept, Reject

    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;

    std::pair<QModelIndex,QModelIndex> addQuery(const IRCPrefix& sender, const QString& description, const std::shared_ptr<DCC>& dcc);
    void removeAll();
    void removeEntry(const QModelIndex& index);

    std::shared_ptr<DCC> getDCCPtr(const QModelIndex& index);
    std::optional<IRCPrefix> getSender(const QModelIndex& index);

private:
    struct Item {
        IRCPrefix sender;
        QString description;
        std::shared_ptr<DCC> dcc;
    };

    std::vector<Item> m_items;
};


#endif //DCCQUERYTABLEMODEL_H
