/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef IDEALIRC_H
#define IDEALIRC_H

#include "MdiManager.h"
#include "IConfig/IConfig.h"
#include "AboutIIRC.h"
#include "DCCQuery.h"
#include "Script/Manager.h"
#include <QMainWindow>
#include <QCloseEvent>
#include <QSystemTrayIcon>
#include <QTimer>
#include <QMenu>

namespace Ui {
class IdealIRC;
}

class IWinStatus;

class IdealIRC : public QMainWindow
{
    Q_OBJECT

public:
    explicit IdealIRC(QWidget* parent = nullptr);
    ~IdealIRC();

private slots:
    void onStatusWindowCreated(IWinStatus* window);
    void onDCCRequested(const IWinStatus& statusParent, const std::shared_ptr<DCC>& dcc, const IRCPrefix& sender, const QString& type, const QString& message);
    void on_actionConnect_triggered();
    void on_actionOptions_triggered();
    void on_actionAbout_IdealIRC_triggered();
    void on_actionScripts_triggered();
    void on_actionOnline_wiki_triggered();
    void on_actionHide_toolbar_triggered();
    void on_actionDCC_queries_triggered();

private:
    enum class ConnectButtonState {
        Connect,
        Disconnect
    };

    void saveSubwindowGeometries();
    void closeEvent(QCloseEvent* evt) override;
    void startup();
    void subwindowSwitched();
    void updateConnectButtonState();
    void setConnectButtonState(ConnectButtonState state);
    void connectToServer(bool newStatus);
    void disconnectFromServer();

    Ui::IdealIRC* ui;
    MdiManager* mdiManager;
    QSystemTrayIcon* trayIcon;
    QMenu* trayIconMenu;
    QAction* trayIconMenuShow;
    QAction* trayIconMenuExit;
    IConfig confDlg;
    AboutIIRC aboutDlg;
    DCCQuery dccQueryDlg;

signals:
    void aboutToClose();
};

#endif // IDEALIRC_H
