/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef MDIMANAGER_H
#define MDIMANAGER_H

#include "IWin/IWin.h"
#include "IRC.h"
#include "ButtonbarMgr.h"
#include <QObject>
#include <QToolBar>
#include <QTreeView>
#include <QMdiArea>
#include <QMdiSubWindow>
#include <QSystemTrayIcon>
#include <QList>
#include <QMenu>
#include <memory>

class IWinStatus;

class MdiManager : public QObject
{
    Q_OBJECT

public:
    MdiManager(QMdiArea& mdiArea, QToolBar& buttonBar, QSystemTrayIcon& trayIcon);
    ~MdiManager();

    static MdiManager& instance();

    IWin* createSubwindow(IWin* parent, const QString& buttonText, IWin::Type windowType, bool activate = true, Highlight buttonHighlight = HL_None);
    IWin* createDCCSubwindow(IWin* parent, IWin::Type windowType, const std::shared_ptr<DCC>& dcc, const IRCPrefix& target);
    IWin* currentWindow() const;
    IWinStatus* currentStatus() const;

    IWin* findWindow(IWin* statusParent, const QString& buttonText);
    QMdiSubWindow* toMdiwin(IWin* win);
    QList<QMdiSubWindow*> mdiChildrenOf(const IWin* statusParent, IWin::Type type = IWin::Type::Undefined) const;
    QList<IWin*> childrenOf(const IWin* statusParent, IWin::Type type = IWin::Type::Undefined) const;
    QList<QMdiSubWindow*> mdiChildren() const;
    void showTrayInfo(const QString& title, const QString& message);
    void showTrayWarn(const QString& title, const QString& message);
    int connectionsOnlineCount() const;
    void broadcastProgramExit();
    void renameWindowButton(IWin* window, const QString& text);

    void print(IWinStatus* parent, const QString& target, const PrintType ptype, const QString& text);
    void print(const QString& target, const PrintType ptype, const QString& text);
    void printToActive(IWinStatus* parent, const PrintType ptype, const QString& text);
    void printToTypes(IWinStatus* parent, const IWin::Type toType, const PrintType ptype, const QString& text);
    void printToAll(IWinStatus* parent, const PrintType ptype, const QString& text);

    void highlight(IWin* window, Highlight state) { m_bbMgr.highlight(window, state); }

private:
    struct ButtonBarEntry {
        ButtonBarEntry(){}
        ButtonBarEntry(QAction* button_, const QString& buttonText, QMdiSubWindow* parent);
        QMdiSubWindow* subwin{ nullptr };
        QAction* button{ nullptr };
        QMenu* menu{ nullptr };
        QAction* menuHead{ nullptr };
        QAction* menuSep{ nullptr };
        QAction* menuClose{ nullptr };
    };
    QMdiArea& m_mdiArea;
    QSystemTrayIcon& trayIcon;
    IWinStatus* m_activeStatus{ nullptr };
    IWin* m_active{ nullptr };
    ButtonbarMgr m_bbMgr;

    IWin* createSubwindowCommons(IWin* parent, IWin* window, const QString& buttonText, const QString& iconPath, bool activate, Highlight buttonHighlight);

    void showTray(const QString& title, const QString& message, QSystemTrayIcon::MessageIcon icon);
    void subwinAboutToClose(IWin* who);
    void subwinActivated(QMdiSubWindow *window);

    int nextXY{ 0 };
    int nextXYadjust{ 0 };
    QRect generateSpawnCoordinates(const QString& networkKey, const QString& windowKey);

    static MdiManager* m_instance;

signals:
    void readyForExit();
    void subwindowSwitched();
    void connectionStateChange();
    void statusWindowCreated(IWinStatus* window);
};

#endif // MDIMANAGER_H
