/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (c) 2022  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "PortMapping.h"
#include <natpmp.h>
#include <chrono>
#include <thread>

namespace {
bool portmapRequest(uint16_t intPort, uint16_t extPort, uint32_t timeout)
{
    natpmp_t request;
    natpmpresp_t response;

    if (initnatpmp(&request, 0, 0) != 0)
        return false;

    auto retry = 10;
    auto res = sendnewportmappingrequest(&request, NATPMP_PROTOCOL_TCP, intPort, extPort, timeout);
    while (res != 12 && retry > 0) {
        if (res == NATPMP_ERR_NOGATEWAYSUPPORT)
            return false;

        std::this_thread::sleep_for( std::chrono::milliseconds(10) );

        res = sendnewportmappingrequest(&request, NATPMP_PROTOCOL_TCP, intPort, extPort, timeout);
        --retry;
    }

    auto readres = readnatpmpresponseorretry(&request, &response);
    retry = 10;
    while (readres != 0 && retry > 0) {
        std::this_thread::sleep_for( std::chrono::milliseconds(10) );
        readres = readnatpmpresponseorretry(&request, &response);
        --retry;
    }

    return retry != 0;
}
}

bool NATPortMapping::add(uint16_t intPort, uint16_t extPort)
{
    constexpr auto SecondsInWeek = 60 * 60 * 24 * 7;
    return portmapRequest(intPort, extPort, SecondsInWeek);
}

bool NATPortMapping::remove(uint16_t intPort, uint16_t extPort)
{
    // Setting new timeout to '0' would cause the port mapping to be removed right away.
    return portmapRequest(intPort, extPort, 0);
}
