/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "Builtin.h"
#include "Script/Script.h"

#include "Mathematics.h"
#include "StringUtils.h"
#include "GeneralUtils.h"
#include "MapUtils.h"
#include "ListUtils.h"
#include "DialogUtils.h"

namespace {
bool alreadyRegistered = false;
}

void registerBuiltinFunctions()
{
    if (alreadyRegistered) return;
    alreadyRegistered = true;

    /*
     * Mathematical functions
    */
    Script::registerUniversalFunction("pow", &Builtin::Mathematics::pow);
    Script::registerUniversalFunction("sqrt", &Builtin::Mathematics::sqrt);
    Script::registerUniversalFunction("cbrt", &Builtin::Mathematics::cbrt);
    Script::registerUniversalFunction("floor", &Builtin::Mathematics::floor);
    Script::registerUniversalFunction("ceil", &Builtin::Mathematics::ceil);
    Script::registerUniversalFunction("round", &Builtin::Mathematics::round);
    Script::registerUniversalFunction("abs", &Builtin::Mathematics::abs);
    Script::registerUniversalFunction("clamp", &Builtin::Mathematics::clamp);
    Script::registerUniversalFunction("clamp01", &Builtin::Mathematics::clamp01);
    Script::registerUniversalFunction("rand", &Builtin::Mathematics::rand);
    Script::registerUniversalFunction("scale", &Builtin::Mathematics::scale);
    Script::registerUniversalFunction("log", &Builtin::Mathematics::log);
    Script::registerUniversalFunction("log10", &Builtin::Mathematics::log10);
    Script::registerUniversalFunction("sin", &Builtin::Mathematics::sin);
    Script::registerUniversalFunction("asin", &Builtin::Mathematics::asin);
    Script::registerUniversalFunction("cos", &Builtin::Mathematics::cos);
    Script::registerUniversalFunction("acos", &Builtin::Mathematics::acos);
    Script::registerUniversalFunction("tan", &Builtin::Mathematics::tan);
    Script::registerUniversalFunction("atan", &Builtin::Mathematics::atan);

    /*
     * String utilities
    */
    Script::registerUniversalFunction("strupper", &Builtin::StringUtils::strupper);
    Script::registerUniversalFunction("strlower", &Builtin::StringUtils::strlower);
    Script::registerUniversalFunction("strlen", &Builtin::StringUtils::strlen);
    Script::registerUniversalFunction("strmid", &Builtin::StringUtils::strmid);
    Script::registerUniversalFunction("strsplit", &Builtin::StringUtils::strsplit);
    Script::registerUniversalFunction("strfind", &Builtin::StringUtils::strfind);
    Script::registerUniversalFunction("strleft", &Builtin::StringUtils::strleft);
    Script::registerUniversalFunction("strright", &Builtin::StringUtils::strright);
    Script::registerUniversalFunction("strrep", &Builtin::StringUtils::strrep);
    Script::registerUniversalFunction("resemblesint", &Builtin::StringUtils::resemblesint);
    Script::registerUniversalFunction("resemblesreal", &Builtin::StringUtils::resemblesreal);
    Script::registerUniversalFunction("resemblesbool", &Builtin::StringUtils::resemblesbool);

    /*
     * General utilities
    */
    Script::registerUniversalFunction("toint", &Builtin::GeneralUtils::toint);
    Script::registerUniversalFunction("tostr", &Builtin::GeneralUtils::tostr);
    Script::registerUniversalFunction("todouble", &Builtin::GeneralUtils::toreal);
    Script::registerUniversalFunction("tobool", &Builtin::GeneralUtils::tobool);
    Script::registerUniversalFunction("isvoid", &Builtin::GeneralUtils::isvoid);
    Script::registerUniversalFunction("isint", &Builtin::GeneralUtils::isint);
    Script::registerUniversalFunction("isreal", &Builtin::GeneralUtils::isreal);
    Script::registerUniversalFunction("isbool", &Builtin::GeneralUtils::isbool);
    Script::registerUniversalFunction("isstr", &Builtin::GeneralUtils::isstr);
    Script::registerUniversalFunction("ismap", &Builtin::GeneralUtils::ismap);
    Script::registerUniversalFunction("decimalsep", &Builtin::GeneralUtils::decimalsep);

    /*
     * Map utilities
    */
    Script::registerUniversalFunction("map", &Builtin::MapUtils::map);
    Script::registerUniversalFunction("maplen", &Builtin::MapUtils::maplen);
    Script::registerUniversalFunction("mapremove", &Builtin::MapUtils::mapremove);
    Script::registerUniversalFunction("mapdump", &Builtin::MapUtils::mapdump);
    Script::registerUniversalFunction("mapkeys", &Builtin::MapUtils::mapkeys);

    /*
     * List utilities
    */
    Script::registerUniversalFunction("list", &Builtin::ListUtils::list);
    Script::registerUniversalFunction("listpush", &Builtin::ListUtils::listpush);
    Script::registerUniversalFunction("listpop", &Builtin::ListUtils::listpop);
    Script::registerUniversalFunction("listlen", &Builtin::ListUtils::listlen);
    Script::registerUniversalFunction("listremove", &Builtin::ListUtils::listremove);
    Script::registerUniversalFunction("listjoin", &Builtin::ListUtils::listjoin);
    Script::registerUniversalFunction("listsplice", &Builtin::ListUtils::listsplice);

    /*
     * Dialog utilities
    */
    Script::registerUniversalFunction("dialog", &Builtin::DialogUtils::dialog);
    Script::registerUniversalFunction("dlgvalidhndl", &Builtin::DialogUtils::dlgvalidhndl);
    Script::registerUniversalFunction("dlgclose", &Builtin::DialogUtils::dlgclose);
    Script::registerUniversalFunction("dlghide", &Builtin::DialogUtils::dlghide);
    Script::registerUniversalFunction("dlgshow", &Builtin::DialogUtils::dlgshow);
    Script::registerUniversalFunction("dlglist", &Builtin::DialogUtils::dlglist);
    Script::registerUniversalFunction("dlgsetattr", &Builtin::DialogUtils::dlgsetattr);
    Script::registerUniversalFunction("dlggetattr", &Builtin::DialogUtils::dlggetattr);
    Script::registerUniversalFunction("dlgsetwidgetattr", &Builtin::DialogUtils::dlgsetwidgetattr);
    Script::registerUniversalFunction("dlggetwidgetattr", &Builtin::DialogUtils::dlggetwidgetattr);
    Script::registerUniversalFunction("dlgtablecount", &Builtin::DialogUtils::dlgtablecount);
    Script::registerUniversalFunction("dlgtableinsert", &Builtin::DialogUtils::dlgtableinsert);
    Script::registerUniversalFunction("dlgtableremove", &Builtin::DialogUtils::dlgtableremove);
    Script::registerUniversalFunction("dlgtableselected", &Builtin::DialogUtils::dlgtableselected);
    Script::registerUniversalFunction("dlgtablerow", &Builtin::DialogUtils::dlgtablerow);
}
