/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef ERROR_H
#define ERROR_H

#include <string>

namespace Builtin::Error {
[[noreturn]] void throwInsufficientParameters(const std::string& fname, size_t given, size_t required);
[[noreturn]] void throwInsufficientParameters(const std::string& fname, size_t given, size_t requiredLow, size_t requiredHigh);
[[noreturn]] void throwNotAString(const std::string& fname, int paramPos);
[[noreturn]] void throwNotANumber(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAStringOrWholeNumber(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAWholeNumber(const std::string& fname, int paramPos);
[[noreturn]] void throwNotADialogHandle(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAnIniHandle(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAMap(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAList(const std::string& fname, int paramPos);
[[noreturn]] void throwIndexOutOfRange(const std::string& fname, int paramPos);
[[noreturn]] void throwNotAReference(const std::string& fname, int paramPos);
[[noreturn]] void throwNegativeInputUndefined(const std::string& fname);
[[noreturn]] void throwNumberMustBePositive(const std::string& fname, int paramPos);
[[noreturn]] void throwUnrecognizedIniHandle(const std::string& fname, int handle);
}

#endif // ERROR_H
