/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef MANAGERLISTMODEL_H
#define MANAGERLISTMODEL_H

#include <QAbstractListModel>
#include <QVector>
#include "config.h"

#if defined(Q_OS_LINUX)
    #define ROOT_FILESYSTEM_CHECK(P) (P[0] == '/')
#elif defined(Q_OS_WIN32) | defined(Q_OS_WIN64)
    #define ROOT_FILESYSTEM_CHECK(P) (P[1] == ':')
#else
    #error Unsupported platform! Update Script/Manager.cpp (MAKE_FIXED_PATH) accordingly.
#endif

#define MAKE_FIXED_PATH(PATH_TO_FILE, FIXED_PATH)   \
    if (!ROOT_FILESYSTEM_CHECK(PATH_TO_FILE))       \
        FIXED_PATH = QStringLiteral("%1/%2").arg(LOCAL_PATH, PATH_TO_FILE)

class ManagerListModel : public QAbstractListModel
{
    Q_OBJECT

public:
    void addPath(const QString& path);
    void delPath(const QString& path);
    QModelIndex pathIndex(const QString& path);

    void markWarning(const QString& entry);
    void unmarkWarning(const QString& entry);

    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
    int rowCount(const QModelIndex& parent = QModelIndex()) const override;

private:
    int findEntry(const QString& entry) const;
    bool isWarning(int idx) const;
    QStringList m_list;
    QVector<int> m_warnings; // entries contain indexes of m_list that is marked with warning.
};

#endif // MANAGERLISTMODEL_H
