/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef MENU_H
#define MENU_H

#include "ScriptException.h"
#include <vector>
#include <memory>
#include <vector>
#include <optional>
#include <string>

class Scope;
class MenuEntry;
struct MenuPriv;
class Menu
{
public:
    Menu() = delete;
    explicit Menu(const std::string& name);
    Menu(Menu&& other);
    Menu(const Menu&) = delete;
    ~Menu();
    Menu& operator=(Menu&& other);
    Menu& operator=(Menu&) = delete;

    const MenuEntry& content() const;
    MenuEntry& content();

    const std::string name() const;

private:
    std::unique_ptr<MenuPriv> mp;
};

using ConstMenuRef = std::reference_wrapper<const Menu>;

struct MenuEntryPriv;
class MenuEntry
{
    friend class Menu;
    friend struct TokenizerPriv;
public:
    MenuEntry();
    MenuEntry(const std::string& label);
    MenuEntry(const std::string& label, Scope&& scope);
    MenuEntry(MenuEntry&& other);
    MenuEntry(const MenuEntry&) = delete;
    MenuEntry& operator=(MenuEntry&) = delete;
    ~MenuEntry();

    enum class Type {
        Item,
        Divide,
        Submenu
    };

    const Scope& scope() const;
    const std::vector<MenuEntry>& entries() const;
    const std::string& label() const;

    MenuEntry& addSubMenu(const std::string& label);
    MenuEntry& addItem(const std::string& label, Scope&& scope);
    MenuEntry& addDivide();
    Type type() const;

private:
    Scope& scope(); // TokenizerPriv uses this
    std::vector<MenuEntry>& entries(); // Menu uses this
    std::unique_ptr<MenuEntryPriv> mp;
};

#endif // MENU_H
