/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef PARSERTOKEN_H
#define PARSERTOKEN_H

#include "ValueHolder.h"
#include "ParserOperator.h"

class TokenBase;

class ParserToken
{
    using Operator = ParserOperator;
public:

    enum class Type
    {
        Value,
        Operator
    };

    ParserToken() = delete;

    ParserToken(ParserToken&& other);

    ParserToken(const ParserToken& other);

    ParserToken(const TokenBase& tokenbase, Operator op);

    ParserToken(const TokenBase& tokenbase, const ValueHolder& val);

    ParserToken& operator=(const ParserToken& other);

    ~ParserToken() = default;

    Type type() const;

    ValueHolder& value();

    const ValueHolder& value() const;

    Operator oper() const;

    static Operator strToOperator(const std::string& oper);

    static int operPrecedence(Operator oper);

    void setOrder(int order);

    int order() const;

    const TokenBase& tokenBase() const;

private:
    int m_order{ 0 };
    std::variant<ValueHolder, Operator> m_tk;
    const TokenBase& m_tokenbase;
};

#endif // PARSERTOKEN_H
