/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "ScriptException.h"

void ScriptException::setMessage()
{
    m_message.push_back('[');
    m_message.append(std::to_string(m_linecol.first));
    m_message.push_back(':');
    m_message.append(std::to_string(m_linecol.second));
    m_message.append("]: ");
    m_message.append(m_originalMessage);
}

ScriptException::ScriptException(const std::string &message)
    : m_originalMessage(message)
{
    setMessage();
}

ScriptException::ScriptException(std::pair<int,int> linecol, const std::string& message)
    : m_linecol(linecol)
    , m_originalMessage(message)
{
    setMessage();
}

ScriptException::ScriptException(std::pair<int, int> linecol, const ScriptException& other)
    : m_linecol(linecol)
    , m_originalMessage(other.m_originalMessage)
{
    setMessage();
}

const char* ScriptException::what() const noexcept
{
    return m_message.c_str();
}
