/*
 * IdealIRC Script Engine - Scripting tailored for IRC clients.
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef TOKENIZER_H
#define TOKENIZER_H

#include "Tokens.h"
#include "Menu.h"
#include "Dialog.h"

#include <string>
#include <optional>
#include <memory>

struct TokenizerPriv;
class Tokenizer
{
public:
    explicit Tokenizer(const std::string& input);
    Tokenizer(Tokenizer&& other);
    Tokenizer() = delete;
    Tokenizer(const Tokenizer&) = delete;
    ~Tokenizer();

    Tokenizer& operator=(const Tokenizer&) = delete;

    bool process();
    const std::string& lastError();
    std::pair<int,int> currentLineCol() const;

    using Iterator = VectorOfUniquePtrsIterator<Scope>;

    Iterator begin() const;
    Iterator end() const;

    const std::vector<std::string>& globals();
    const std::vector<Menu>& menus() const;
    const std::vector<Dialog>& dialogs() const;

private:
    /* TODO Consider moving the below functions into TokenizerPriv */
    bool nextKeyword();
    bool processImperativeBlock(ImperativeBlockType blockType);
    bool processGlobals();
    bool processMenu();
    bool processDialog();
    /* *** */

    std::unique_ptr<TokenizerPriv> mp;
    const std::string m_data; // maybe move this one too, to TokenizerPriv?
};

#endif // TOKENIZER_H
