/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#ifndef SCRIPTDIALOG_H
#define SCRIPTDIALOG_H

/** @class ScriptDialog
 * Qt / IdealIRC representation of a scripted dialog.
 * This holds an actual instance of a dialog.
*/

#include <QObject>
#include "Script/Dialog.h"
#include <memory>

class Script;

struct ScriptDialogPriv;
class ScriptDialog : public QObject
{
    Q_OBJECT

public:
    ScriptDialog(const Dialog& dialog, Script& script, int handle);
    ScriptDialog() = delete;
    ScriptDialog(const ScriptDialog&) = delete;
    ScriptDialog(ScriptDialog&& other);
    ~ScriptDialog();
    ScriptDialog& operator=(ScriptDialog&) = delete;

    void setAttr(const std::string& key, ValueHolder& value);
    ValueHolder attr(const std::string& key) const;

    void setWidgetAttr(const std::string& widgetName, const std::string& key, ValueHolder& value);
    ValueHolder widgetAttr(const std::string& widgetName, const std::string& key) const;

    int tableWidgetCount(const std::string& widgetName);
    void tableWidgetInsert(const std::string& widgetName, const ValueArray& cols);
    void tableWidgetRemove(const std::string& widgetName, int index);
    int tableWidgetSelected(const std::string& widgetName);
    ValueArray tableWidgetRow(const std::string& widgetName, int index);

    const std::string& name() const;

    void close();
    void hide();
    void show();

private:
    std::unique_ptr<ScriptDialogPriv> mp;
};

#endif // SCRIPTDIALOG_H
