/*
 * IdealIRC - Internet Relay Chat client
 * Copyright (C) 2021  Tom-Andre Barstad.
 * This software is licensed under the Software Attribution License.
 * See LICENSE for more information.
*/

#include "ScriptIRCUtils.h"
#include "Script/Builtin/Error.h"
#include "Script/ValueExtract.h"
#include "Script/ScriptException.h"
#include "Script/Manager.h"
#include "IWin/IWinStatus.h"
#include "IRC.h"
#include "IRCClient/IRCMember.h"

using namespace Builtin::Error;

namespace ScriptFunctions {

ValueHolder userhost(Script&, std::vector<ValueHolder>& args)
{
    if (args.size() != 1)
        throwInsufficientParameters("userhost", args.size(), 1);

    if (args[0].getType() != ValueHolder::Type::String)
        throwNotAString("userhost", 1);

    const auto& connection = ScriptManager::getContext()->getConnection();
    const auto nickname = args[0].toStdString();
    const auto member = connection.getMember(nickname);

    if (!member || member->prefix().host().empty())
        return ValueHolder();

    return member->prefix().composite();
}

ValueHolder isupport(Script&, std::vector<ValueHolder>& args)
{
    if (args.size() != 1)
        throwInsufficientParameters("isupport", args.size(), 1);

    if (args[0].getType() != ValueHolder::Type::String)
        throwNotAString("isupport", 1);

    const auto& connection = ScriptManager::getContext()->getConnection();
    const auto& isupport = connection.isupport();
    try {
        return isupport.at(args[0].toStdString());
    } catch (const std::out_of_range&) {
        return ValueHolder();
    }
}

}
